import csv
from dataclasses import field
from threading import local
from venv import create 
from tqdm import tqdm
import re
import time
import requests
from datetime import datetime
from ambra_sdk.api import Api
import json
from tkinter import Tk
from tkinter import filedialog

"""

Study thin add template -- Ensure column headers match values in the studyadd function

"""

def studyadd(study):
    global adds
    if 'namespace' in globals() and namespace not in ['',None]:
        study['storage_namespace'] = namespace
        study['phi_namespace'] = namespace
    if 'node' in globals() and node not in ['',None]:
        study['node_id'] = node
    try:
    
        request = {
            "URL":"/study/add",
            "sid":sid,
            "storage_namespace":study.get("storage_namespace"),
            "phi_namespace":study.get("phi_namespace"),
            "accession_number":study.get("accession_number"),
            "modality":study.get('modality'),
            "patient_birth_date":study.get("patient_birth_date"),
            "patient_name":study.get("patient_name"),
            "patient_sex":study.get("patient_sex"),
            "patientid":study.get("patientid"),
            "referring_physician":study.get("referring_physician"),
            "study_description":study.get("study_description"),
            "study_date":study.get("study_date"),
            "study_uid":study.get("study_uid"),
            "node_id":study.get("node_id"),
            'thin':1
        }
        return request
    except Exception as err:
        print(err)



    

# csv.field_size_limit(50000<<10) for really big csvs increase the csv python memory limit


if __name__ =='__main__':
    sid = input("sid: ")
    api = Api.with_sid('https://access.ambrahealth.com/api/v3', sid)
    namespace = input("namespace: ")
    node = input("node: ")
    Tk()

    thin_data_filename = filedialog.askopenfile(title="select thin add csv file").name
    existing_uid_filename = filedialog.askopenfile(title="existing uid csv file").name
    lines = csv.DictReader(open(thin_data_filename,"r+",encoding="utf-8-sig"))
    uids = set(l['study_uid'] for l in csv.DictReader(open(existing_uid_filename,'r')))
    #Get the length of the file
    length=0
    for l in open(thin_data_filename,"r+"): 
        length+=1

    #if historic_data gets too big add counter to reset after size gets to 
    historic_data = {}
    studies_to_add=0
    print("processing data...")
    for row in tqdm(lines): 
        historic_data[row['study_uid']] = row

    
    historic_uid = set(historic_data.keys())
    new_uid = list(historic_uid-uids)

    chunk = []
    print("adding studies...")
    for uid in tqdm(new_uid,total =len(new_uid)):
        study = historic_data[uid]
        print(uid)
        if len(chunk) < 25:
            req = studyadd(study)
            if req != None:
                chunk.append(req)
        else:
            chunk.append(req)
            try:
                res = requests.post("https://access.ambrahealth.com/api/v3/bundle",data = json.dumps(chunk))
                response = res.json()
            except:
                time.sleep(30)          
                res = requests.post("https://access.ambrahealth.com/api/v3/bundle",data = json.dumps(chunk))
                response = res.json()         
            chunk = []
    if len(chunk)>0:
        res = requests.post("https://access.ambrahealth.com/api/v3/bundle",data = json.dumps(chunk))
        response = res.json()
        chunk = []



#from time import sleep
# from ambra_sdk.api import Api 
import requests
#from re import findall, match
import json
#import re
# import platformAPI as papi

# sdk = papi.create_ambra_sdk_api()
# sid = sdk.sid

#sid = '' 

"""
    
Ethan York 
September 14, 2022 at 10:06 AM

copy viewer3_config FROM a user (setting/get)
TO account
TO Role

TO another user


    Create the following custom fields in TS scripts
    user_uuid


"""


def custom_code(data):
        
        key = 'viewer3_config'

        environment = "access"
        url = "https://" + environment + ".ambrahealth.com/api/v3/"
        
        customfields = {i["name"]:i["value"] for i in data["study"]["customfields"]}
        
        api_call_data = {
            'sid': sid,
            'user_id': customfields["original_user_uuid"],
            'key': key
        }

        


        response = requests.post(url + "setting/get", data=api_call_data).json()
        response['value'] = response['value'].encode('raw_unicode_escape').decode('unicode_escape')

        pass
        
        #copy to role
        if customfields['role_uuid'] not in ["", None]:
            api_call_data['uuid'] = customfields['role_uuid']
            api_call_data["permission_viewer3_config"] = response["value"]
            role_response = requests.post(url + "role/set", data=api_call_data).json()
            
        
        #copy to account
        if customfields['account_uuid'] not in ["", None]:
            api_call_data['uuid'] = customfields['account_uuid']
            api_call_data["setting_viewer3_config"] = response["value"] 
            acct_response = requests.post(url + "account/set", data=api_call_data).json()

        if customfields['other_user_uuid'] not in ["", None]:
            api_call_data['user_id'] = customfields['other_user_uuid']
            api_call_data['value'] = response["value"]
            user_response = requests.post(url + 'setting/set', data=api_call_data)

        


#custom_code(data={"user_uuid":'94c5882d-76f5-484c-9615-6db2bf543684', 'role_uuid':'bbe9313d-884a-4cd8-b017-b25cfa8a08b6',
#'account_uuid':"7c94e43a-e890-409e-96ae-25df89b014b7", "other_user_uuid":"a74eefc2-59e4-41ed-baa6-8a15d07fc171"})


import requests
import json 

url = input("enter url (default if blank https://access.ambrahealth.com/api/v3): ")
if url == "":
    url = "https://access.ambrahealth.com/api/v3"
account_id = input("enter account id: ")
sid = input("enter sid: ")
#
setting_to_change = input("enter setting to change (ex: allow_upload_share_code_groups_locations): ")
setting_value= input("enter setting value: ")

account = requests.post(url+"/account/get",data={"sid":sid,"uuid":account_id}).json()
ui_json = json.loads(account['settings']['ui_json'])
ui_json[setting_to_change] = setting_value

update = requests.post(url+"/account/set",data={"sid":sid,"uuid":account_id,"setting_ui_json":json.dumps(ui_json)}).json()
print(update)


import csv
# from tqdm import tqdm
import time
import requests
import json
from tkinter import Tk
from tkinter import filedialog
from multiprocessing.pool import Pool
"""

Study thin add template -- Ensure column headers match values in the studyadd function

"""
#namespace = '09070acd-43ed-4ccc-a861-6f9eba443e99'
#node = 'd5e98b1d-c320-45c1-87a6-eb7a3591e9a9'

#accn_set = {}

csv.field_size_limit(50000<<10) #for really big csvs increase the csv python memory limit

def send_requests_bundle(reqs,retry=0):
    failed = 0
    ok = 0
    url = "https://access.ambrahealth.com/api/v3"
    try:
        response = requests.post(url+"/bundle",data=json.dumps(reqs)).json()
        for r in response:
            if r['status'] != "OK":
                failedwriter = csv.DictWriter(open('failed.csv','a',newline=""),fieldnames=['uuid','reason'])
                index = response.index(r)
                failed_study = reqs[index]
                failedwriter.writerow({"uuid":failed_study.get('study_id'),"reason":r['error_type']})
                failed+=1
            else:
                ok+=1
        return [ok,failed]
    except Exception as err:
        if retry < 3:
            time.sleep(10)
            retry+=1
            send_requests_bundle(reqs, retry)
        else:
            return


if __name__ =='__main__':
    
    sid = input("sid: ")
    share_type = input("share to type: account_id or location_id or group_id or user_id or share_code or email: ")
    share_id = input(share_type+": ")
    namespace = "09070acd-43ed-4ccc-a861-6f9eba443e99"
    node = "d5e98b1d-c320-45c1-87a6-eb7a3591e9a9"
    root = Tk()
    # root.withdraw()
    study_list_file = filedialog.askopenfilename(title="csv list of studies with uuid header")
    studies = set(l['uuid'] for l in csv.DictReader(open(study_list_file,'r')))
    print("processing data...")
    bundles = []
    reqs = []
    for uuid in studies: 
        reqs.append(
            {"URL":"/study/share",
            "uuid":uuid,
            "sid":sid,
            share_type:share_id
             }
            )
        if len(reqs)==100:
            bundles.append(reqs)
            reqs = []
            
    bundles.append(reqs)
    pool = Pool(processes=10)
    results = []
    ok = 0
    fail = 0
    print("total:"+str(len(studies)))
    for r in pool.imap_unordered(send_requests_bundle, bundles):
        ok+=r[0]
        fail+=r[1]
        print("number of OK api responses: "+str(ok)+"-- number of failed shares: " +str(fail), end="\r")
